/*
 * The MIT License
 *
 *   Copyright (c) 2020, Mahmoud Ben Hassine (mahmoud.benhassine@icloud.com)
 *
 *   Permission is hereby granted, free of charge, to any person obtaining a copy
 *   of this software and associated documentation files (the "Software"), to deal
 *   in the Software without restriction, including without limitation the rights
 *   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *   copies of the Software, and to permit persons to whom the Software is
 *   furnished to do so, subject to the following conditions:
 *
 *   The above copyright notice and this permission notice shall be included in
 *   all copies or substantial portions of the Software.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *   THE SOFTWARE.
 */
package org.jeasy.batch.xml;

import org.jeasy.batch.core.record.Record;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.xml.sax.SAXParseException;

import java.nio.file.Paths;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class XmlRecordValidatorTest {

    @Mock
    private Record<Foo> record;

    private XmlRecordValidator<Foo> xmlRecordValidator;

    @Before
    public void setUp() throws Exception {
        xmlRecordValidator = new XmlRecordValidator<>(Paths.get("src/test/resources/foo.xsd"));
    }

    @Test
    public void whenRecordIsValid_thenShouldReturnTheSameRecord() throws Exception {
        // given
        Foo Foo = new Foo("foo"); // valid name: length = 3 chars <= expected max 5 chars
        when(record.getPayload()).thenReturn(Foo);

        // when
        Record<Foo> fooRecord = xmlRecordValidator.processRecord(record);

        // then
        assertThat(fooRecord).isEqualTo(record);
    }

    @Test(expected = SAXParseException.class)
    public void whenRecordIsNotValid_thenShouldReturnThrowAnException() throws Exception {
        // given
        Foo Foo = new Foo("foobar"); // invalid name: length = 6 chars > expected max 5 chars
        when(record.getPayload()).thenReturn(Foo);

        // when
        xmlRecordValidator.processRecord(record);

        // then
        // excepted exception
    }
}
